#
class apt(
  $update   = {},
  $purge    = {},
  $proxy    = {},
  $sources  = {},
  $keys     = {},
  $ppas     = {},
  $pins     = {},
  $settings = {},
) inherits ::apt::params {

  # See rant regarding lookups and merging in Readme
  if (empty($sources) or !$sources) {
    # no hiera, no direct param
    $all_sources=undef
  } elsif (! empty (hiera_hash('apt::sources', {}))) {
    # sources comes from hiera => override
    $all_sources = hiera_hash('apt::sources')
  } else {
    # sources comes as direct parameter: leave alone
    $all_sources=$sources
  }

  # See rant regarding lookups and merging in Readme
  if (empty($settings) or ! $settings) {
    # no hiera, no direct param
    $all_settings=undef
  } elsif (! empty (hiera_hash('apt::settings', {}))) {
    # settings comes from hiera => override
    $all_settings = hiera_hash('apt::settings')
  } else {
    # settings comes as direct parameter: leave alone
    $all_settings=$settings
  }

  # See rant regarding lookups and merging in Readme
  if (empty($keys) or ! $keys) {
    # no hiera, no direct param
    $all_keys={}
  } elsif (! empty (hiera_hash('apt::keys', {}))) {
    # keys comes from hiera => override
    $all_keys = hiera_hash('apt::keys')
  } else {
    # keys comes as direct parameter: leave alone
    $all_keys=$keys
  }

  # See rant regarding lookups and merging in Readme
  if (empty($ppas) or ! $ppas) {
    # no hiera, no direct param
    $all_ppas={}
  } elsif (! empty (hiera_hash('apt::ppas', {}))) {
    # ppas comes from hiera => override
    $all_ppas = hiera_hash('apt::ppas')
  } else {
    # ppas comes as direct parameter: leave alone
    $all_ppas=$ppas
  }

  # See rant regarding lookups and merging in Readme
  if (empty($pins) or ! $pins) {
    # no hiera, no direct param
    $all_pins={}
  } elsif (! empty (hiera_hash('apt::pins', {}))) {
    # pins comes from hiera => override
    $all_pins = hiera_hash('apt::pins')
  } else {
    # pins comes as direct parameter: leave alone
    $all_pins=$pins
  }

  $frequency_options = ['always','daily','weekly','reluctantly']
  validate_hash($update)
  if $update['frequency'] {
    validate_re($update['frequency'], $frequency_options)
  }
  if $update['timeout'] {
    unless is_integer($update['timeout']) {
      fail('timeout value for update must be an integer')
    }
  }
  if $update['tries'] {
    unless is_integer($update['tries']) {
      fail('tries value for update must be an integer')
    }
  }

  $_update = merge($::apt::update_defaults, $update)
  include ::apt::update

  validate_hash($purge)
  if $purge['sources.list'] {
    validate_bool($purge['sources.list'])
  }
  if $purge['sources.list.d'] {
    validate_bool($purge['sources.list.d'])
  }
  if $purge['preferences'] {
    validate_bool($purge['preferences'])
  }
  if $purge['preferences.d'] {
    validate_bool($purge['preferences.d'])
  }

  $_purge = merge($::apt::purge_defaults, $purge)

  validate_hash($proxy)
  if $proxy['ensure'] {
    validate_re($proxy['ensure'], ['file', 'present', 'absent'])
  }
  if $proxy['host'] {
    validate_string($proxy['host'])
  }
  if $proxy['port'] {
    unless is_integer($proxy['port']) {
      fail('$proxy port must be an integer')
    }
  }
  if $proxy['https'] {
    validate_bool($proxy['https'])
  }

  $_proxy = merge($apt::proxy_defaults, $proxy)

  validate_hash($all_sources)
  validate_hash($all_keys)
  validate_hash($all_settings)
  validate_hash($all_ppas)
  validate_hash($all_pins)

  if $_proxy['ensure'] == 'absent' or $_proxy['host'] {
    apt::setting { 'conf-proxy':
      ensure   => $_proxy['ensure'],
      priority => '01',
      content  => template('apt/_conf_header.erb', 'apt/proxy.erb'),
    }
  }

  $sources_list_content = $_purge['sources.list'] ? {
    true    => "# Repos managed by puppet.\n",
    default => undef,
  }

  $preferences_ensure = $_purge['preferences'] ? {
    true    => absent,
    default => file,
  }

  if $_update['frequency'] == 'always' {
    Exec <| title=='apt_update' |> {
      refreshonly => false,
    }
  }

  apt::setting { 'conf-update-stamp':
    priority => 15,
    content  => template('apt/_conf_header.erb', 'apt/15update-stamp.erb'),
  }

  file { 'sources.list':
    ensure  => file,
    path    => $::apt::sources_list,
    owner   => root,
    group   => root,
    mode    => '0644',
    content => $sources_list_content,
    notify  => Class['apt::update'],
  }

  file { 'sources.list.d':
    ensure  => directory,
    path    => $::apt::sources_list_d,
    owner   => root,
    group   => root,
    mode    => '0644',
    purge   => $_purge['sources.list.d'],
    recurse => $_purge['sources.list.d'],
    notify  => Class['apt::update'],
  }

  file { 'preferences':
    ensure => $preferences_ensure,
    path   => $::apt::preferences,
    owner  => root,
    group  => root,
    mode   => '0644',
    notify => Class['apt::update'],
  }

  file { 'preferences.d':
    ensure  => directory,
    path    => $::apt::preferences_d,
    owner   => root,
    group   => root,
    mode    => '0644',
    purge   => $_purge['preferences.d'],
    recurse => $_purge['preferences.d'],
    notify  => Class['apt::update'],
  }

  # manage sources if present
  if $all_sources {
    create_resources('apt::source', $all_sources)
  }
  # manage keys if present
  if $all_keys {
    create_resources('apt::key', $all_keys)
  }
  # manage ppas if present
  if $all_ppas {
    create_resources('apt::ppa', $all_ppas)
  }
  # manage settings if present
  if $all_settings {
    create_resources('apt::setting', $all_settings)
  }

  # manage pins if present
  if $all_pins {
    create_resources('apt::pin', $all_pins)
  }
}
